﻿Imports System.Drawing
Imports System.Windows.Forms

Public Class WebsitePicture
    Private _uriString As String
    ''' <summary>
    ''' Legt die Adresse der Seite fest.
    ''' </summary>
    Public Property uriString() As String
        Get
            Return _uriString
        End Get
        Set(ByVal value As String)
            _uriString = value

            If Not value.StartsWith("http://") Then
                _uriString = "http://" & value
            Else
                _uriString = value
            End If
        End Set
    End Property

    ''' <summary>
    ''' Die uriString im Uri-Format.
    ''' </summary>
    Public ReadOnly Property uri() As Uri
        Get
            Return New Uri(Me.uriString)
        End Get
    End Property

    Private _imageSize As Size
    ''' <summary>
    ''' Legt die Grösse des Bildes fest.
    ''' </summary>
    Public Property imageSize() As Size
        Get
            Return _imageSize
        End Get
        Set(ByVal value As Size)
            _imageSize = value

            'Browser vergrössern.
            _wB.Size = value
        End Set
    End Property

    'Neues WebBrowser-Control.
    Private WithEvents _wB As New WebBrowser

    'Die Stopuhr.
    Private _sW As New Stopwatch

    'Variable zum Speichern vom Status.
    Private _status As Status

    'Das Bitmap.
    Private _b As Bitmap

    'Die Events :-) Danke Spatzenkanonier!
    Public Event ImageReady As EventHandler(Of WebsitePictureEventArgs)
    Public Event ProgressChanged As EventHandler(Of WebsitePicutureProgressEventArgs)

    ''' <summary>
    ''' Status festlegen.
    ''' </summary>
    Private Enum Status
        Active = 0
        Idle = 1
    End Enum

    ''' <summary>
    ''' Konstruktor mit Defaultwerten. uriString = "", imageSize = 1024*768.
    ''' </summary>
    Public Sub New()
        Me.New("", New Size(1024, 768))
    End Sub

    ''' <summary>
    ''' Konstruktor mit URL. imageSize = 1024*768.
    ''' </summary>
    Public Sub New(ByVal url As String)
        Me.New(url, New Size(1024, 768))
    End Sub

    ''' <summary>
    ''' Konstruktor mit URL und definierter imageSize.
    ''' </summary>
    Public Sub New(ByVal url As String, ByVal previewSize As Size)
        Me.uriString = url
        Me.imageSize = previewSize

        _wB.ScriptErrorsSuppressed = True
        _wB.ScrollBarsEnabled = False
        _wB.Size = previewSize
    End Sub

    ''' <summary>
    ''' Löst das Event FullImageCreated aus.
    ''' </summary>
    Public Sub GetFullImage()
        'Aktivieren.
        _status = Status.Active

        'Stopuhr zurücksetzen und neu starten.
        _sW.Reset()
        _sW.Start()

        'Navigation wieder erlauben.
        _wB.AllowNavigation = True

        'WebBrowser laden lassen.
        _wB.Navigate(Me.uri)
    End Sub

    ''' <summary>
    ''' Wird ausgelöst, wenn die Webseite fertig geladen ist.
    ''' </summary>
    Private Sub _wB_DocumentCompleted(ByVal sender As Object, ByVal e As WebBrowserDocumentCompletedEventArgs) Handles _wB.DocumentCompleted
        'Wenn idle, nix machen.
        If _status = Status.Idle Then Exit Sub

        'Stopuhr anhalten.
        _sW.Stop()

        'Browser anhalten.
        _wB.Stop()

        'Automatische Weiterleitungen verhindern.
        _wB.AllowNavigation = False

        'Das _wB-Control zu Control casten, da _wB kein DrawToBitmap besitzt.
        Dim tempCtrl As Control = CType(_wB, Control)

        'Bitmap erzeugen.
        _b = New Bitmap(Me.imageSize.Width, Me.imageSize.Height)

        'Das _wB-Control in das Bitmap zeichnen.
        tempCtrl.DrawToBitmap(_b, New Rectangle(0, 0, _b.Width, _b.Height))

        'Event auslösen.
        OnImageReady(New WebsitePictureEventArgs(_b, _sW.Elapsed))

        _b = Nothing
        tempCtrl = Nothing

        'Benötigt, um nicht unbeabsichtigt ein Event auszulösen.
        _status = Status.Idle
    End Sub

    ''' <summary>
    ''' Erzeugt das Event.
    ''' </summary>
    Protected Overridable Sub OnImageReady(ByVal e As WebsitePictureEventArgs)
        RaiseEvent ImageReady(Me, e)
    End Sub

    ''' <summary>
    ''' Erzeugt das Changed-Event.
    ''' </summary>
    Protected Overridable Sub OnProgressChanged(ByVal e As WebsitePicutureProgressEventArgs)
        RaiseEvent ProgressChanged(Me, e)
    End Sub

    Private Sub _wB_ProgressChanged(ByVal sender As Object, ByVal e As System.Windows.Forms.WebBrowserProgressChangedEventArgs) Handles _wB.ProgressChanged
        OnProgressChanged(New WebsitePicutureProgressEventArgs(_sW.Elapsed))
    End Sub
End Class
